var _StoragePlugin = function () {

    var tinymce = top && top.tinymce ? top.tinymce : null,
        activeEditor = (tinymce && tinymce.activeEditor) ? tinymce.activeEditor : null,
        windowManager = (activeEditor && activeEditor.windowManager) ? activeEditor.windowManager : null,
        params = windowManager ? windowManager.getParams() || {} : {},
        url = params.url || '',
        storageUrl = params.storageUrl || '',
        tokenName = params.tokenName || '',
        tokenValue = params.tokenValue || '',
        previewWidth = parseInt(params.previewWidth) || 30,
        previewHeight = parseInt(params.previewHeight) || 30,
        previewMode = params.previewMode || 'outbound',
        previewQuality = parseInt(params.previewQuality) || 100,
        getUrl = function () {
            var parts = arguments || [],
                result = [];
            for (var i = 0; i < parts.length; i++) {
                var part = parts[i];
                if (part) {
                    if (part[0] === '/') {
                        part = part.substr(1);
                    }
                    if (part[part.length - 1] === '/') {
                        part = part.substr(0, part.length - 1);
                    }
                    result.push(part);
                }
            }
            return result.join('/');
        },
        getData = function (url, data) {
            url = url || '';
            data = data || {};
            var defer = $.Deferred(),
                headers = [],
                requestData = [];
            for (var d in data) {
                if (data.hasOwnProperty(d)) {
                    requestData.push(d + '=' + data[d]);
                }
            }
            url = url + '?' + encodeURI(requestData.join('&'));
            headers.push({key: tokenName, value: tokenValue});
            tinymce.util.XHR.send({
                type: 'GET',
                url: url,
                requestheaders: headers,
                success: function (responseText) {
                    var response = tinymce.util.JSON.parse(responseText);
                    if (response && response.data) {
                        defer.resolve(response.data);
                    } else {
                        defer.reject(response);
                    }
                },
                error: function (responseText) {
                    var response = tinymce.util.JSON.parse(responseText);
                    defer.reject(response);
                }
            });

            return defer;
        };

    return {
        /**
         * Закрывает окно выбора изображения
         */
        closeWindow: function () {
            windowManager && windowManager.close();
        },
        /**
         * Вставляет выбранное изображение в текущую позицию документа
         * @param path Путь категории
         * @param name Имя выбранного файла
         */
        insertImage: function (path, name) {
            var src = getUrl(storageUrl, 'attach', path, name);
            activeEditor && activeEditor.insertContent('<img src="' + src + '" />');
        },
        /**
         * Преобразовывает trail в путь
         * @param trail
         * @returns {string}
         */
        trail2path: function (trail) {
            trail = trail || '';
            return trail.split('.').join('/');
        },
        /**
         * Возвращает ссылку на предпросмотр изображения
         * @param imageGuid Идентификатор изображения
         */
        getPreviewImageUrl: function (imageGuid) {
            return getUrl(storageUrl, 'images/scale', imageGuid, previewWidth, previewHeight, previewMode, previewQuality);
        },
        /**
         * Возвращает список категорий изображений
         * @param page Номер страницы
         * @param perPage Количество записей на страницу
         * @param sort Параметры сортировки
         * @returns {*}
         */
        getScopes: function (page, perPage, sort) {
            page = page || 1;
            perPage = perPage || 20;
            sort = sort || 'created';
            var data = {page: page, 'per-page': perPage, sort: sort};
            return getData(getUrl(url, 'scopes'), data);
        },
        /**
         * Возвращает список изображений из указанной категории
         * @param path Путь к категории
         * @param page Номер страницы
         * @param perPage Количество записей на страницу
         * @param sort Параметры сортировки
         */
        getScopeAttachments: function (path, page, perPage, sort) {
            path = path || '';
            page = page || 1;
            perPage = perPage || 20;
            sort = sort || 'created';
            var data = {page: page, 'per-page': perPage, sort: sort};
            return getData(getUrl(url, 'scope-attachments', path, 'attach'), data);
        }
    };
};
