import {
	ChangeDetectionStrategy,
	Component,
	Input,
	OnChanges,
	SimpleChanges,
} from '@angular/core';
import { ButtonClasses } from './model/buttonClasses.model';

@Component({
	selector: 'gp-button',
	templateUrl: './gp-button.component.html',
	styleUrls: ['./gp-button.component.scss'],
	changeDetection: ChangeDetectionStrategy.OnPush
})
export class GpButtonComponent implements OnChanges {

	// Тип
	@Input() public type = '';

	// Цвет
	@Input() public color = '';

	// Дизейбл
	@Input() public disabled = false;

	// Дефолтный класс
	private readonly defaultClass = 'gp-button';

	// Цвета, Типы
	private readonly classes = new ButtonClasses(this.defaultClass);

	// Присваемый набор классов к кнопке
	public currentClass = this.defaultClass;

	// Создаем список классов
	private setClass(changes: SimpleChanges): string {
		const disabledClass: string = changes.disabled ? this.classes.disabled : this.classes.none;
		const colorClass: (keyof ButtonClasses) = changes.color ? changes.color.currentValue : this.classes.primary;
		const typeClass: (keyof ButtonClasses) = changes.type ? changes.type.currentValue : this.classes.basic;

		return `${this.defaultClass} ${this.classes[colorClass]} ${this.classes[typeClass]} ${disabledClass}`;
	}

	// В зависимости от переданных данных в компонент, присваиваем нужные классы
	public ngOnChanges(changes: SimpleChanges): void {
		this.currentClass = this.setClass(changes);
	}
}
