import { Component, Input, OnInit } from '@angular/core';
import {
	ControlValueAccessor,
	FormBuilder,
	FormControl,
	FormGroup,
	NG_VALUE_ACCESSOR
} from '@angular/forms';

@Component({
	selector: 'gp-checkbox',
	templateUrl: './gp-checkbox.component.html',
	styleUrls: ['./gp-checkbox.component.scss'],
	providers: [
		{
			provide: NG_VALUE_ACCESSOR,
			useExisting: GpCheckboxComponent,
			multi: true
		}
	]
})
export class GpCheckboxComponent implements ControlValueAccessor, OnInit {

	// Состояние ошибки
	@Input() public error = false;

	// Тайтл чекбокса
	@Input() public title = '';

	// Дефолтный класс формы
	public defaultClass = 'gp-checkbox__form';

	// Набор классов для формы
	public formClass = this.defaultClass;

	// Конструктор форм
	public formBuilder: FormBuilder = new FormBuilder();

	// Генерируем форму для дальнейшей подписки на события
	public formGroup: FormGroup = this.formBuilder.group({
		checkbox: new FormControl()
	});

	// Храним состояние чекбокса
	public inputChecked: boolean | string = false;

	// Храним в переменной доступ к чекбоксу
	public readonly checkbox = this.formGroup.get('checkbox') as FormControl;

	public ngOnInit(): void {
		// Подписка на события чекбокса
		this.subscribeToCheckboxEvents();
	}

	// Подписываемся на изменения событий, Уведомляем родителя, что данные изменились
	private subscribeToCheckboxEvents(): void {
		if (this.checkbox) {
			this.checkbox
				.valueChanges
				.subscribe((data: string) => {
					this.inputChecked = data;
					this.propagateChange(data);
				});
		}
	}

	// Записываем данные переданные от родителя
	public writeValue(value: boolean): void {
		if (this.checkbox) {
			this.checkbox.setValue(value);
		}
	}

	// Необходимые методы для работы с ngModel
	public registerOnChange(fn: any): void {
		this.propagateChange = fn;
	}

	// tslint:disable-next-line:no-empty
	public propagateChange = (_: any): void => {};

	// tslint:disable-next-line:no-empty
	public registerOnTouched(fn: any): void {}
}
